import simplejson, base64

from django.db.models.manager import Manager
from django.core.cache import cache

from django_roa.db.query import RemoteQuerySet


class CacheQuerySet(RemoteQuerySet):
    def filter(self, *args, **kwargs):
        opts = self.model._meta
        
        key = '%s.%s.%s' % (opts.app_label, opts.module_name, simplejson.dumps(kwargs))
        key = base64.b64encode(key)

        obj = cache.get(key)
        if obj is not None:
            self._result_cache = obj
        else:
            self._result_cache = super(CacheQuerySet, self).filter(*args, **kwargs)
            cache.set(key, self._result_cache)

        return self._result_cache

class ROAManager(Manager):
    """
    Manager which access remote resources.
    """
    use_for_related_fields = True
    is_roa_manager = True # ugly but useful because isinstance is evil
    
    def get_query_set(self):
        """
        Returns a QuerySet which access remote resources.
        """
        return CacheQuerySet(self.model)
